/*
  Simple Recursion Example.  This code is reliant on a separate process to call it multiple times; the main focus here is to show how to use Limit (as it is NOT like SQL's Limit) and LastEvaluatedKey (for allowing a successive call to pick up where the previous call stopped) by passing it to ExclusiveStartKey.
*/

var AWS = require("aws-sdk");
var docClient = new AWS.DynamoDB.DocumentClient();

exports.handler = function(event, context) {
	/*
	In this example the column "UniqueID" for all rows of a table is retrieved.  The column "rowPartitionID" is the unique partition key (aka primary key of the table).
	"LastEvaluatedKey" is used to hold the last row DynamoDB has examined (and is passed in on the next call to this function so the next set of "UniqueID" can be retrieved); in this case the column is "rowPartitionID".
	If the value of "LastEvaluatedKey" returned is 0 then there are no further rows to iterate in the table.
	*/
	Ids = [];
	var numIDsReturnedPerCall = 1000;	/* The number of rows to examine for matches at a time (this is NOT the same as how limit is used in SQL) */
	var LastEvaluatedKey = 0;
	/* Get argument passed to this function */
	if (event.LastEvaluatedKey) {
		if (parseInt(event.LastEvaluatedKey) > 0) {
			LastEvaluatedKey = parseInt(event.LastEvaluatedKey);
		}
	}
	/* Determine what type of query to use */
	var tmp_rowPartitionID = 0; /* In this example the unique table partition id column contains a unique value (like a primary key in SQL) */
	if (LastEvaluatedKey === 0) {
		/* Initial call */
		params = {
			TableName: "TableOfIDs",
			Limit: numIDsReturnedPerCall,
			ProjectionExpression: "UniqueID",
			FilterExpression: "rowPartitionID > :datainrowPartitionID",
			ExpressionAttributeValues: {
				":datainrowPartitionID": tmp_rowPartitionID
			}
		};
	}
	else {
		/* Follow-up call so start where last call left off (pagination) */
		/* Note: Because "ExclusiveStartKey" expects a key:value, and to keep it simple in evaluating what is passed into this function in the event of recusion (LastEvaluatedKey as an integer), create that key:value syntax for DynamoDB (remember that the LastEvaluatedKey, depending on how the table is setup, could be a partition key:value AND a sort key:value) */
		var keyval_LastEvaluatedKey = {
			"rowPartitionID": LastEvaluatedKey
		};
		params = {
			TableName: "TableOfIDs",
			Limit: numIDsReturnedPerCall,
			ExclusiveStartKey: keyval_LastEvaluatedKey,
			ProjectionExpression: "UniqueID",
			FilterExpression: "rowPartitionID > :datainrowPartitionID",
			ExpressionAttributeValues: {
				":datainrowPartitionID": tmp_rowPartitionID
			}
		};
	}
	/* Scan the table.  "Query" statement cannot be used here because Query only recognizes an equality statement for the unique partition key (aka primary key) and sort key (if used).  In this case we have just a partition key range of values to pull back ALL rows with a partition id value greater than 0 */
	docClient.scan(params, function(err, data) {
		if (err) {
			/* There's a problem with the database request */
			context.done(
				null,
				{
				"error_msg": JSON.stringify(err, null, 2)
				}
			);
		}
		else {
			/* We have a response from the databse request */
			var Uid = "";
			data.Items.forEach(function(row) {
				Uid = row.UniqueID;
				if (Uid) {
					Ids[Ids.length] = {
						"uid": Uid
					};
				}
			});
			if (typeof data.LastEvaluatedKey !== "undefined" && typeof data.LastEvaluatedKey !== null) {
				/* NOTE: By default "data.LastEvaluatedKey" will return a key:value instead of the actual value of a single key (remember it is possible that a partition key/value and a sort key/value could be returned depending on how a table is setup).  If LastEvaluatedKey has no data then the end of the table has been reached */
				LastEvaluatedKey = data.LastEvaluatedKey.rowPartitionID;
			}
			else { LastEvaluatedKey = 0; }
			/* Generate Response */
			context.done(
				null,
				{
				"Ids": Ids,
				"LastEvaluatedKey": LastEvaluatedKey
				}
			);
		}
	};
};